{==================================================================
  QddgDBSpin Unit

  This unit implements the TddgDBSpinner component, which
  illustrates how to convert a non-data-aware component into a
  data-aware version.

  Copyright  2001 by Ray Konopka
==================================================================}

unit QddgDBSpin;

interface

uses
  SysUtils, Classes, Qt, QddgILSpin, DB, QDBCtrls;
  (*
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  ddgILSpin, DB, DBCtrls;
  *)

type
  TddgDBSpinner = class( TddgImgListSpinner )
  private
    FDataLink: TFieldDataLink;        // Provides Access to Data

    // Internal Event Handlers for DataLink Events
    procedure DataChange( Sender: TObject );
    procedure UpdateData( Sender: TObject );
    procedure ActiveChange( Sender: TObject );

    (*
    // VCL->CLX:  Component Message handling methods not in CLX
    procedure CMExit( var Msg: TCMExit ); message cm_Exit;
    procedure CMDesignHitTest( var Msg: TCMDesignHitTest );
      message cm_DesignHitTest;
    *)
  protected
    procedure Notification( AComponent : TComponent;
                            Operation : TOperation ); override;
    procedure CheckFieldType( const Value: string ); virtual;

    // Overridden event dispatch methods
    procedure Change; override;
    procedure KeyPress( var Key : Char ); override;

    // VCL->CLX:  DoExit replaces CMExit
    procedure DoExit; override;
    // VCL->CLX:  DesignEventQuery replaces CMDesignHitTest
    function DesignEventQuery( Sender: QObjectH;
                               Event: QEventH ): Boolean; override;

    // Overridden support methods
    procedure DecValue( Amount: Integer ); override;
    procedure IncValue( Amount: Integer ); override;

    // Property Access Methods
    function GetField: TField; virtual;
    function GetDataField: string; virtual;
    procedure SetDataField( const Value: string ); virtual;
    function GetDataSource: TDataSource; virtual;
    procedure SetDataSource( Value: TDataSource ); virtual;
    function GetReadOnly: Boolean; virtual;
    procedure SetReadOnly( Value: Boolean ); virtual;

    // Give Descendants Access to Field object and DataLink
    property Field: TField
      read GetField;

    property DataLink: TFieldDataLink
      read FDataLink;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
  published
    property DataField: string
      read GetDataField
      write SetDataField;

    property DataSource: TDataSource
      read GetDataSource
      write SetDataSource;

    // This property controls the ReadOnly state of the DataLink
    property ReadOnly: Boolean
      read GetReadOnly
      write SetReadOnly
      default False;
  end;

type
  EInvalidFieldType = class( Exception );

resourcestring
  SInvalidFieldType = 'DataField can only be connected to ' +
                      'columns of type Integer, Smallint, Word, ' +
                      'and Float';

implementation

uses
  Types;                        // VCL->CLX:  Added for CLS support


{===========================}
{== TddgDBSpinner Methods ==}
{===========================}

constructor TddgDBSpinner.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );

  FDataLink := TFieldDataLink.Create;

  // To support the TField.FocusControl method, set the
  // FDataLink.Control property to point to the spinner.
  // The Control property requires a TWinControl component.
  FDataLink.Control := Self;

  // Assign Event Handlers
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
  FDataLink.OnActiveChange := ActiveChange;

  // NOTE: Since, the component user does not have direct access to
  // the data link, the user cannot assign custom event handlers.
end;


destructor TddgDBSpinner.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;


procedure TddgDBSpinner.Notification( AComponent: TComponent;
                                     Operation: TOperation );
begin
  inherited Notification( AComponent, Operation );
  if ( Operation = opRemove ) and
     ( FDataLink <> nil ) and
     ( AComponent = FDataLink.DataSource ) then
  begin
    DataSource := nil;            // Indirectly calls SetDataSource
  end;
end;


function TddgDBSpinner.GetField: TField;
begin
  Result := FDataLink.Field;
end;


function TddgDBSpinner.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TddgDBSpinner.SetDataField( const Value: string );
begin
  CheckFieldType( Value );
  FDataLink.FieldName := Value;
end;


function TddgDBSpinner.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TddgDBSpinner.SetDataSource( Value: TDataSource );
begin
  if FDatalink.DataSource <> Value then
  begin
    FDataLink.DataSource := Value;

    // FreeNotification must be called b/c DataSource may be
    // located on another form or data module.
    if Value <> nil then
      Value.FreeNotification( Self );
  end;
end;


function TddgDBSpinner.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TddgDBSpinner.SetReadOnly( Value: Boolean );
begin
  FDataLink.ReadOnly := Value;
end;



procedure TddgDBSpinner.CheckFieldType( const Value: string );
var
  FieldType: TFieldType;
begin
  // Make sure the field type corresponding to the column
  // referenced by Value is either ftInteger, ftSmallInt, ftWord,
  // or ftFloat.  If it is not, an EInvalidFieldType exception is
  // raised.

  if ( Value <> '' ) and
     ( FDataLink <> nil ) and
     ( FDataLink.Dataset <> nil ) and
     ( FDataLink.Dataset.Active ) then
  begin
    FieldType := FDataLink.Dataset.FieldByName( Value ).DataType;
    if ( FieldType <> ftInteger ) and
       ( FieldType <> ftSmallInt ) and
       ( FieldType <> ftWord ) and
       ( FieldType <> ftFloat ) then
    begin
      raise EInvalidFieldType.Create( SInvalidFieldType );
    end;
  end;
end;


procedure TddgDBSpinner.Change;
begin
  // Tell the FDataLink that the data has changed
  if FDataLink <> nil then
    FDataLink.Modified;
  inherited Change;                     // Generates OnChange event
end;


procedure TddgDBSpinner.KeyPress( var Key: Char );
begin
  inherited KeyPress( Key );

  if Key = #27 then
  begin
    FDataLink.Reset;                             // Esc key pressed
    Key := #0;               // Set to #0 so Esc won't close dialog
  end;
end;


procedure TddgDBSpinner.DecValue( Amount: Integer );
begin
  if ReadOnly or not FDataLink.CanModify then
  begin
    // Prevent change if FDataLink is ReadOnly
    (*
    // VCL->CLX:  MessageBeep is a Windows API function
    MessageBeep( 0 )
    *)
    Beep;
  end
  else
  begin
    // Try to put Dataset in edit mode--only dec if in edit mode
    if FDataLink.Edit then
      inherited DecValue( Amount );
  end;
end;


procedure TddgDBSpinner.IncValue( Amount: Integer );
begin
  if ReadOnly or not FDataLink.CanModify then
  begin
    // Prevent change if FDataLink is ReadOnly
    (*
    // VCL->CLX:  MessageBeep is a Windows API function
    MessageBeep( 0 )
    *)
    Beep;
  end
  else
  begin
    // Try to put Dataset in edit mode--only inc if in edit mode
    if FDataLink.Edit then
      inherited IncValue( Amount );
  end;
end;


{==================================================================
  TddgDBSpinner.DataChange

  This method gets called as a result of a number of
  different events:

  1. The underlying field value changes.  Occurs when changing the
     value of the column tied to this control and then move to a
     new column or a new record.
  2. The corresponding Dataset goes into Edit mode.
  3. The corresponding Dataset referenced by DataSource changes.
  4. The current cursor is scrolled to a new record in the table.
  5. The record is reset through a Cancel call.
  6. The DataField property changes to reference another column.
==================================================================}

procedure TddgDBSpinner.DataChange( Sender: TObject );
begin
  if FDataLink.Field <> nil then
    Value := FDataLink.Field.AsInteger;
end;


{==================================================================
  TddgDBSpinner.UpdateData

  This method gets called when the corresponding field value and
  the contents of the Spinner need to be synchronized.  Note that
  this method only gets called if this control was responsible for
  altering the data.
==================================================================}

procedure TddgDBSpinner.UpdateData( Sender: TObject );
begin
  FDataLink.Field.AsInteger := Value;
end;


{==================================================================
  TddgDBSpinner.ActiveChange

  This method gets called whenever the Active property of the
  attached Dataset changes.

  NOTE: You can use the FDataLink.Active property to determine
        the *new* state of the Dataset.
==================================================================}

procedure TddgDBSpinner.ActiveChange( Sender: TObject );
begin
  // If the Dataset is becoming Active, then check to make sure the
  // field type of the DataField property is a valid type.

  if ( FDataLink <> nil ) and FDataLink.Active then
    CheckFieldType( DataField );
end;


(*
// VCL->CLX:  CMExit replaced with DoExit (see below)

procedure TddgDBSpinner.CMExit( var Msg: TCMExit );
begin
  try   // Attempt to update the record if focus leaves the spinner
    FDataLink.UpdateRecord;
  except
    SetFocus;      // Keep the focus on the control if Update fails
    raise;                                 // Reraise the exception
  end;
  inherited;
end;
*)

procedure TddgDBSpinner.DoExit;
begin
  try   // Attempt to update the record if focus leaves the spinner
    FDataLink.UpdateRecord;
  except
    SetFocus;      // Keep the focus on the control if Update fails
    raise;                                 // Reraise the exception
  end;
  inherited;
end;


(*
// VCL->CLX:  CMDesignHitTest replaced by DesignEventQuery

procedure TddgDBSpinner.CMDesignHitTest(var Msg: TCMDesignHitTest);
begin
  // Ancestor component allows Value to be changed at design-time.
  // This is not valid in a data-aware component because it would
  // put the connected dataset into edit mode.
  Msg.Result := 0;
end;
*)

function TddgDBSpinner.DesignEventQuery( Sender: QObjectH;
                                        Event: QEventH ): Boolean;
begin
  // Ancestor component allows Value to be changed at design-time.
  // This is not valid in a data-aware component because it would
  // put the connected dataset into edit mode.
  Result := False;
end;

end.
