{==================================================================
  QddgILSpin Unit

  This unit implements the TddgImgListSpinner component, which
  descends from TddgDesignSpinner and allows the user to connect an
  image list to the spinner and specify image indexes for the plus
  and minus buttons.

  Copyright  2001 by Ray Konopka
==================================================================}

unit QddgILSpin;

interface

uses
  Classes, Types, QddgSpin, QddgDsnSpn, QImgList;
  (*
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  ddgSpin, ddgDsnSpn, ImgList;
  *)

type
  TddgImgListSpinner = class( TddgDesignSpinner )
  private
    FImages: TCustomImageList;
    FImageIndexes: array[ 1..2 ] of Integer;
    FImageChangeLink: TChangeLink;

    // Internal Event Handlers
    procedure ImageListChange( Sender: TObject );
  protected
    procedure Notification( AComponent : TComponent;
                            Operation : TOperation ); override;

    procedure DrawButton( Button: TddgButtonType; Down: Boolean;
                          Bounds: TRect ); override;
    procedure CalcCenterOffsets( Bounds: TRect; var L, T: Integer);

    procedure CheckMinSize;

    // Property Access Methods
    procedure SetImages( Value: TCustomImageList ); virtual;
    function GetImageIndex( PropIndex: Integer ): Integer; virtual;
    procedure SetImageIndex( PropIndex: Integer;
                             Value: Integer ); virtual;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
  published
    property Images: TCustomImageList
      read FImages
      write SetImages;

    property ImageIndexMinus: Integer
      index 1
      read GetImageIndex
      write SetImageIndex;

    property ImageIndexPlus: Integer
      index 2
      read GetImageIndex
      write SetImageIndex;
  end;

implementation

uses
  QGraphics;                    // VCL->CLX:  Added for CLX support


{================================}
{== TddgImgListSpinner Methods ==}
{================================}

constructor TddgImgListSpinner.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );

  FImageChangeLink := TChangeLink.Create;
  FImageChangeLink.OnChange := ImageListChange;
  // NOTE: Since, the component user does not have direct access to
  // the change link, the user cannot assign custom event handlers.

  FImageIndexes[ 1 ] := -1;
  FImageIndexes[ 2 ] := -1;
end;


destructor TddgImgListSpinner.Destroy;
begin
  FImageChangeLink.Free;
  inherited Destroy;
end;


procedure TddgImgListSpinner.Notification( AComponent: TComponent;
                                          Operation: TOperation );
begin
  inherited Notification( AComponent, Operation );
  if ( Operation = opRemove ) and ( AComponent = FImages ) then
    SetImages( nil );             // Note the call to access method
end;


function TddgImgListSpinner.GetImageIndex( PropIndex:
                                          Integer ): Integer;
begin
  Result := FImageIndexes[ PropIndex ];
end;

procedure TddgImgListSpinner.SetImageIndex( PropIndex: Integer;
                                           Value: Integer );
begin
  if FImageIndexes[ PropIndex ] <> Value then
  begin
    FImageIndexes[ PropIndex ] := Value;
    Invalidate;
  end;
end;


procedure TddgImgListSpinner.SetImages( Value: TCustomImageList );
begin
  if FImages <> nil then
    FImages.UnRegisterChanges( FImageChangeLink );

  FImages := Value;

  if FImages <> nil then
  begin
    FImages.RegisterChanges( FImageChangeLink );
    FImages.FreeNotification( Self );
    CheckMinSize;
  end;
  Invalidate;
end;

procedure TddgImgListSpinner.ImageListChange( Sender: TObject );
begin
  if Sender = Images then
  begin
    CheckMinSize;
    // Call Update instead of Invalidate to prevent flicker
    Update;
  end;
end;

procedure TddgImgListSpinner.CheckMinSize;
begin
  // Ensures button area will display entire image
  if FImages.Width > ButtonWidth then
    ButtonWidth := FImages.Width;
  if FImages.Height > Height then
    Height := FImages.Height;
end;


procedure TddgImgListSpinner.DrawButton( Button: TddgButtonType;
                                        Down: Boolean;
                                        Bounds: TRect );
var
  L, T: Integer;
begin
  with Canvas do
  begin
    Brush.Color := ButtonColor;
    Pen.Color := clBtnShadow;
    Rectangle( Bounds.Left, Bounds.Top,
               Bounds.Right, Bounds.Bottom );

    if Button = btMinus then           // Draw the Minus (-) Button
    begin
      if ( Images <> nil ) and ( ImageIndexMinus <> -1 ) then
      begin
        (*
        // VCL->CLX:  DrawingStyle does not exist in CLX TImageList
        //            BkColor is used instead.
        if Down then
          FImages.DrawingStyle := dsSelected
        else
          FImages.DrawingStyle := dsNormal;
        *)
        if Down then
          FImages.BkColor := clBtnShadow
        else
          FImages.BkColor := clBtnFace;

        CalcCenterOffsets( Bounds, L, T );

        (*
        // VCL->CLX:  TImageList.Draw is different in CLX
        FImages.Draw( Canvas, L, T, ImageIndexMinus, Enabled );
        *)
        FImages.Draw( Canvas, L, T, ImageIndexMinus, itImage,
                      Enabled );
      end
      else
        inherited DrawButton( Button, Down, Bounds );
    end
    else                                // Draw the Plus (+) Button
    begin
      if ( Images <> nil ) and ( ImageIndexPlus <> -1 ) then
      begin
        (*
        // VCL->CLX:  DrawingStyle does not exist in CLX TImageList
        //            BkColor is used instead.
        if Down then
          FImages.DrawingStyle := dsSelected
        else
          FImages.DrawingStyle := dsNormal;
        *)
        if Down then
          FImages.BkColor := clBtnShadow
        else
          FImages.BkColor := clBtnFace;

        CalcCenterOffsets( Bounds, L, T );

        (*
        // VCL->CLX:  TImageList.Draw is different in CLX
        FImages.Draw( Canvas, L, T, ImageIndexPlus, Enabled );
        *)
        FImages.Draw( Canvas, L, T, ImageIndexPlus, itImage,
                      Enabled );
      end
      else
        inherited DrawButton( Button, Down, Bounds );
    end;
  end;
end; {= TddgImgListSpinner.DrawButton =}


procedure TddgImgListSpinner.CalcCenterOffsets( Bounds: TRect;
                                               var L, T: Integer );
begin
  if FImages <> nil then
  begin
    L := Bounds.Left + ( Bounds.Right - Bounds.Left ) div 2 -
         ( FImages.Width div 2 );
    T := Bounds.Top + ( Bounds.Bottom - Bounds.Top ) div 2 -
         ( FImages.Height div 2 );
  end;
end;

end.
