{==================================================================
  QddgSpin Unit

  This unit implements the TddgSpinner custom CLX component, which
  displays the increment and decrement buttons that change the
  spinner's value at each end of the spinner rather than on top of
  one another at one end.

  This component descends from the CLX version of TCustomControl
  and demonstrates many features of component building such as
  managing keyboard focus, custom painting, and mouse processing.

  Copyright  2001 by Ray Konopka
==================================================================}

unit QddgSpin;

interface

uses
  SysUtils, Classes, Types, Qt, QControls, QGraphics;
  (*
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  ImgList;
  *)

type
  TddgButtonType = ( btMinus, btPlus );
  TddgSpinnerEvent = procedure (Sender: TObject; NewValue: Integer;
                              var AllowChange: Boolean ) of object;

  TddgSpinner = class( TCustomControl )
  private
    // Instance Data for Component
    FValue: Integer;
    FIncrement: Integer;
    FButtonColor: TColor;
    FButtonWidth: Integer;
    FMinusBtnDown: Boolean;
    FPlusBtnDown: Boolean;

    // Method Pointers to Hold Custom Events
    FOnChange: TNotifyEvent;
    FOnChanging: TddgSpinnerEvent;

    (*
    // VCL->CLX:  These message handlers are not available in CLX

    // Window Message Handling Method
    procedure WMGetDlgCode( var Msg: TWMGetDlgCode );
      message wm_GetDlgCode;

    // Component Message Handling Method
    procedure CMEnabledChanged( var Msg: TMessage );
      message cm_EnabledChanged;
    *)
  protected
    procedure Paint; override;
    procedure DrawButton( Button: TddgButtonType; Down: Boolean;
                          Bounds: TRect ); virtual;

    // Support Methods
    procedure DecValue( Amount: Integer ); virtual;
    procedure IncValue( Amount: Integer ); virtual;

    function CursorPosition: TPoint;
    function MouseOverButton( Btn: TddgButtonType ): Boolean;

    // VCL->CLX:  EnabledChanged replaces cm_EnabledChanged
    //            component message handler
    procedure EnabledChanged; override;

    // New Event Dispatch Methods
    procedure Change; dynamic;
    function CanChange( NewValue: Integer ): Boolean; dynamic;

    // Overridden Event Dispatch Methods
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;

    procedure MouseDown( Button: TMouseButton; Shift: TShiftState;
                         X, Y: Integer ); override;
    procedure MouseUp( Button: TMouseButton; Shift: TShiftState;
                       X, Y: Integer ); override;

    (*
    // VCL->CLX:  These following declarations have changed in CLX

    function DoMouseWheelDown( Shift: TShiftState;
                             MousePos: TPoint ): Boolean; override;

    function DoMouseWheelUp( Shift: TShiftState;
                             MousePos: TPoint ): Boolean; override;
    *)

    function DoMouseWheelDown( Shift: TShiftState;
                       const MousePos: TPoint ): Boolean; override;

    function DoMouseWheelUp( Shift: TShiftState;
                       const MousePos: TPoint ): Boolean; override;


    // Access Methods for Properties
    procedure SetButtonColor( Value: TColor ); virtual;
    procedure SetButtonWidth( Value: Integer ); virtual;
    procedure SetValue( Value: Integer ); virtual;
  public
    // Don't forget to specify override for constructor
    constructor Create( AOwner: TComponent ); override;
  published
    // New Property Declarations
    property ButtonColor: TColor
      read FButtonColor
      write SetButtonColor
      default clBtnFace;

    property ButtonWidth: Integer
      read FButtonWidth
      write SetButtonWidth
      default 18;

    property Increment: Integer
      read FIncrement
      write FIncrement
      default 1;

    property Value: Integer
      read FValue
      write SetValue;

    // New Event Declarations

    property OnChange: TNotifyEvent
      read FOnChange
      write FOnChange;

    property OnChanging: TddgSpinnerEvent
      read FOnChanging
      write FOnChanging;

    // Inherited Properties and Events
    property Color;
    (*
    property DragCursor;      // VCL->CLX:  Property not yet in CLX
    *)
    property DragMode;
    property Enabled;
    property Font;
    property Height default 18;
    property HelpContext;
    property Hint;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property Width default 80;

    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;


implementation


{=========================}
{== TddgSpinner Methods ==}
{=========================}

constructor TddgSpinner.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );

  // Initialize Instance Data
  FButtonColor := clBtnFace;
  FButtonWidth := 18;
  FValue := 0;
  FIncrement := 1;

  FMinusBtnDown := False;
  FPlusBtnDown := False;

  // Initializing inherited properties
  Width := 80;
  Height := 18;
  TabStop := True;

  // VCL->CLX:  TWidgetControl sets Color property to clNone
  Color := clWindow;

  // VCL->CLX:  InputKeys assignment replaces handling the
  //            wm_GetDlgCode message.
  InputKeys := InputKeys + [ ikArrows ];
end;


{== Property Access Methods ==}

procedure TddgSpinner.SetButtonColor( Value: TColor );
begin
  if FButtonColor <> Value then
  begin
    FButtonColor := Value;
    Invalidate;
  end;
end;

procedure TddgSpinner.SetButtonWidth( Value: Integer );
begin
  if FButtonWidth <> Value then
  begin
    FButtonWidth := Value;
    Invalidate;
  end;
end;


procedure TddgSpinner.SetValue( Value: Integer );
begin
  if FValue <> Value then
  begin
    if CanChange( Value ) then
    begin
      FValue := Value;
      Invalidate;

      // Trigger Change event
      Change;
    end;
  end;
end;


{== Painting Related Methods ==}

procedure TddgSpinner.Paint;
var
  R: TRect;
  YOffset: Integer;
  S: string;
  XOffset: Integer;             // VCL->CLX:  Added for CLX support
begin
  inherited Paint;
  with Canvas do
  begin
    Font := Self.Font;
    Pen.Color := clBtnShadow;

    if Enabled then
      Brush.Color := Self.Color
    else
    begin
      Brush.Color := clBtnFace;
      Font.Color := clBtnShadow;
    end;

    // Display Value
    (*
    // VCL->CLX:  SetTextAlign not available in CLX
    SetTextAlign( Handle, ta_Center or ta_Top );    // GDI function
    *)

    R := Rect( FButtonWidth - 1, 0,
               Width - FButtonWidth + 1, Height );
    Canvas.Rectangle( R.Left, R.Top, R.Right, R.Bottom );
    InflateRect( R, -1, -1 );


    S := IntToStr( FValue );
    YOffset := R.Top + ( R.Bottom - R.Top -
                         Canvas.TextHeight( S ) ) div 2;

    // VCL->CLX:  Calculate XOffset b/c no SetTextAlign function
    XOffset := R.Left + ( R.Right - R.Left -
                          Canvas.TextWidth( S ) ) div 2;

    (*
    // VCL->CLX:  Change TextRect call b/c no SetTextAlign function
    TextRect( R, Width div 2, YOffset, S );
    *)
    TextRect( R, XOffset, YOffset, S );

    DrawButton( btMinus, FMinusBtnDown,
                Rect( 0, 0, FButtonWidth, Height ) );
    DrawButton( btPlus, FPlusBtnDown,
                Rect( Width - FButtonWidth, 0, Width, Height ) );

    if Focused then
    begin
      Brush.Color := Self.Color;
      DrawFocusRect( R );
    end;
  end;
end; {= TddgSpinner.Paint =}


procedure TddgSpinner.DrawButton( Button: TddgButtonType;
                                  Down: Boolean; Bounds: TRect );
begin
  with Canvas do
  begin
    if Down then                            // Set background color
      Brush.Color := clBtnShadow
    else
      Brush.Color := FButtonColor;
    Pen.Color := clBtnShadow;
    Rectangle( Bounds.Left, Bounds.Top,
               Bounds.Right, Bounds.Bottom );

    if Enabled then
    begin
      (*
      // VCL->CLX: clActiveCaption is set to
      //           clActiveHighlightedText in CLX.
      Pen.Color := clActiveCaption;
      Brush.Color := clActiveCaption;
      *)
      Pen.Color := clActiveBorder;
      Brush.Color := clActiveBorder;
    end
    else
    begin
      Pen.Color := clBtnShadow;
      Brush.Color := clBtnShadow;
    end;

    if Button = btMinus then               // Draw the Minus Button
    begin
      Rectangle( 4, Height div 2 - 1,
                 FButtonWidth - 4, Height div 2 + 1 );
    end
    else                                    // Draw the Plus Button
    begin
      Rectangle( Width - FButtonWidth + 4, Height div 2 - 1,
                 Width - 4, Height div 2 + 1 );
      Rectangle( Width - FButtonWidth div 2 - 1,
                 ( Height div 2 ) - (FButtonWidth div 2 - 4),
                 Width - FButtonWidth div 2 + 1,
                 ( Height div 2 ) + (FButtonWidth div 2 - 4)  );
    end;
    Pen.Color := clWindowText;
    Brush.Color := clWindow;
  end;
end; {= TddgSpinner.DrawButton =}


procedure TddgSpinner.DoEnter;
begin
  inherited DoEnter;
  // Controls gets focus--update display to show focus border
  Repaint;
end;

procedure TddgSpinner.DoExit;
begin
  inherited DoExit;
  // Control lost focus--update display to remove focus border
  Repaint;
end;


// VCL->CLX:  EnabledChanged replaces cm_EnabledChanged handler

procedure TddgSpinner.EnabledChanged;
begin
  inherited;
  // Repaint the component so that it reflects the state change
  Repaint;
end;


{== Event Dispatch Methods ==}

{==================================================================
  TddgSpinner.CanChange

  This is the event dispatch method supporting the OnChanging
  event. Notice that this method is a function, rather than the
  common procedure variety. As a function, the Result variable is
  assigned a value before calling the user defined event handler.
==================================================================}

function TddgSpinner.CanChange( NewValue: Integer ): Boolean;
var
  AllowChange: Boolean;
begin
  AllowChange := True;
  if Assigned( FOnChanging ) then
    FOnChanging( Self, NewValue, AllowChange );
  Result := AllowChange;
end;


procedure TddgSpinner.Change;
begin
  if Assigned( FOnChange ) then
    FOnChange( Self );
end;


// Notice that both DecValue and IncValue assign the new value to
// the Value property (not FValue), which indirectly calls SetValue

procedure TddgSpinner.DecValue( Amount: Integer );
begin
  Value := Value - Amount;
end;

procedure TddgSpinner.IncValue( Amount: Integer );
begin
  Value := Value + Amount;
end;


{== Keyboard Processing Methods ==}

(*
// VCL->CLX:  Replaced with InputKeys assignment in constructor

procedure TddgSpinner.WMGetDlgCode( var Msg: TWMGetDlgCode );
begin
  inherited;
  Msg.Result := dlgc_WantArrows;  // Control will handle arrow keys
end;
*)


procedure TddgSpinner.KeyDown( var Key: Word; Shift: TShiftState );
begin
  inherited KeyDown( Key, Shift );

  // VCL->CLX:  Key constants changed in CLX.
  //            vk_ prefix changed to Key_

  case Key of
    Key_Left, Key_Down:
      DecValue( FIncrement );

    Key_Up, Key_Right:
      IncValue( FIncrement );
  end;
end;


{== Mouse Processing Methods ==}

function TddgSpinner.CursorPosition: TPoint;
begin
  GetCursorPos( Result );
  Result := ScreenToClient( Result );
end;


function TddgSpinner.MouseOverButton(Btn: TddgButtonType): Boolean;
var
  R: TRect;
begin
  // Get bounds of appropriate button
  if Btn = btMinus then
    R := Rect( 0, 0, FButtonWidth, Height )
  else
    R := Rect( Width - FButtonWidth, 0, Width, Height );

  // Is cursor position within bounding rectangle?
  Result := PtInRect( R, CursorPosition );
end;


procedure TddgSpinner.MouseDown( Button: TMouseButton;
                                Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown( Button, Shift, X, Y );

  if not ( csDesigning in ComponentState ) then
    SetFocus;              // Move focus to Spinner only at runtime

  if ( Button = mbLeft ) and
     ( MouseOverButton(btMinus) or MouseOverButton(btPlus) ) then
  begin
    FMinusBtnDown := MouseOverButton( btMinus );
    FPlusBtnDown := MouseOverButton( btPlus );

    Repaint;
  end;
end;


procedure TddgSpinner.MouseUp( Button: TMouseButton;
                              Shift: TShiftState; X, Y: Integer );
begin
  inherited MouseUp( Button, Shift, X, Y );

  if Button = mbLeft then
  begin
    if MouseOverButton( btPlus ) then
      IncValue( FIncrement )
    else if MouseOverButton( btMinus ) then
      DecValue( FIncrement );

    FMinusBtnDown := False;
    FPlusBtnDown := False;

    Repaint;
  end;
end;


function TddgSpinner.DoMouseWheelDown( Shift: TShiftState;
                                 const MousePos: TPoint ): Boolean;
begin
  inherited DoMouseWheelDown( Shift, MousePos );
  DecValue( FIncrement );
  Result := True;
end;


function TddgSpinner.DoMouseWheelUp( Shift: TShiftState;
                                 const MousePos: TPoint ): Boolean;
begin
  inherited DoMouseWheelUp( Shift, MousePos );
  IncValue( FIncrement );
  Result := True;
end;

end.
